package com.ruoyi.web.controller.warehouse.finance;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.utils.spring.SpringUtils;
import com.ruoyi.finance.domain.TFee;
import com.ruoyi.finance.domain.TWareHouseFees;
import com.ruoyi.finance.service.ITFeeService;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.warehouseBusiness.domain.TWareHouseExcelItem;
import com.ruoyi.warehouseBusiness.domain.TWareHouseExcelItems;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * 财务对账数据Controller
 *
 * @author ruoyi
 * @date 2021-01-18
 */
@RestController
@RequestMapping("/finance/contrast")
public class TContrastController extends BaseController {
    @Autowired
    private ITFeeService tFeeService;

    /**
     * 查询财务数据主列表
     */
    @PreAuthorize("@ss.hasPermi('finance:contrast:list')")
    @GetMapping("/list")
    public TableDataInfo list(TFee tFee) {
        tFee.setfBilltype("DZ");
        startPage();
        List<Map<String, Object>> list = tFeeService.selectTFeeList1(tFee);
        return getDataTable(list);
    }

    /**
     * 导出财务数据主列表
     */
    @PreAuthorize("@ss.hasPermi('finance:contrast:export')")
    @Log(title = "财务数据主", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TFee tFee) {
        List<TFee> list = tFeeService.selectTFeeList(tFee);
        ExcelUtil<TFee> util = new ExcelUtil<TFee>(TFee.class);
        return util.exportExcel(list, "fee");
    }

    /**
     * 获取财务数据主详细信息
     */
    @PreAuthorize("@ss.hasPermi('finance:contrast:query')")
    @GetMapping(value = "/{fId}")
    public AjaxResult getInfo(@PathVariable("fId") Long fId) {
        return AjaxResult.success(tFeeService.selectTFeeById(fId));
    }

    /**
     * 新增财务数据主
     */
    @PreAuthorize("@ss.hasPermi('finance:contrast:add')")
    @Log(title = "财务数据主", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    @RepeatSubmit
    public AjaxResult add(@RequestParam("tFee") String tFee,
                          @RequestParam("tFeeDo") String tFeeDo ) {
        String billsType = "DZ";
        // 获取当前的用户
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return tFeeService.insertTFeeTFeeDo(tFee,tFeeDo,loginUser,billsType);
    }

    /**
     * 修改财务数据主
     */
    @PreAuthorize("@ss.hasPermi('finance:contrast:edit')")
    @Log(title = "财务数据主", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TFee tFee) {
        return toAjax(tFeeService.updateTFee(tFee));
    }

    /**
     * 删除财务数据主
     */
    @PreAuthorize("@ss.hasPermi('finance:contrast:remove')")
    @Log(title = "财务数据主", businessType = BusinessType.DELETE)
    @DeleteMapping("/{fIds}")
    public AjaxResult remove(@PathVariable Long[] fIds) {
        return toAjax(tFeeService.deleteTFeeByIds(fIds));
    }

    @PreAuthorize("@ss.hasPermi('finance:contrast:remove')")
    @Log(title = "财务数据主", businessType = BusinessType.DELETE)
    @DeleteMapping("/queryRemove/{fId}")
    public AjaxResult queryRemove(@PathVariable("fId") Long fId) {
        return tFeeService.queryRemove(fId);
    }

    /**
     *  收费查询
     */
    @PreAuthorize("@ss.hasPermi('finance:contrast:list')")
    @GetMapping("/contrastList")
    public TableDataInfo chargeList(TWareHouseFees tWareHouseFees) {
        startPage();
        String billsType = "DZ";
        List<Map<String, Object>> list = tFeeService.warehouseBillsFeesList(tWareHouseFees,billsType);
        return getDataTable(list);
    }

    /**
     *   确认对账
     */
    @PreAuthorize("@ss.hasPermi('finance:contrast:add')")
    @Log(title = "财务数据主", businessType = BusinessType.INSERT)
    @PostMapping(value = "/confirm")
    @RepeatSubmit
    public AjaxResult confirm(@RequestParam("tFee") String tFee,
                              @RequestParam("tFeeDo") String tFeeDo ) {
        String billsType = "DZ";
        // 获取当前的用户
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return tFeeService.confirm(tFee,tFeeDo,loginUser,billsType);
    }

    /**
     *   撤销对账
     */
    @PreAuthorize("@ss.hasPermi('finance:contrast:add')")
    @Log(title = "财务数据主", businessType = BusinessType.INSERT)
    @PostMapping(value = "/revoke")
    @RepeatSubmit
    public AjaxResult revoke(@RequestParam("tFee") String tFee,
                             @RequestParam("tFeeDo") String tFeeDo ) {
        String billsType = "DZRevoke";
        // 获取当前的用户
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return tFeeService.revoke(tFee,tFeeDo,loginUser,billsType);
    }

    /**
     * 导出入库详情主表列表
     */
    @GetMapping("/exportItems/{fId}")
    public AjaxResult exportItems(@PathVariable("fId") Long fId) {
        List<TWareHouseExcelItem> list = tFeeService.selectTWarehousebillsItemsList(fId);
        String exportName = tFeeService.findCtrlcorpid(fId);
        ExcelUtil<TWareHouseExcelItem> util = new ExcelUtil<TWareHouseExcelItem>(TWareHouseExcelItem.class);
        return util.exportExcel(list, exportName);
    }
}
