package com.ruoyi.basicData.mapper;


import com.ruoyi.basicData.domain.TWarehouseArea;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.TWarehouse;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;

/**
 * 仓库Mapper接口
 *
 * @author ruoyi
 * @date 2020-12-11
 */
public interface TWarehouseMapper {
    /**
     * 查询仓库
     *
     * @param fId 仓库ID
     * @return 仓库
     */
    public TWarehouse selectTWarehouseById(Long fId);

    /**
     * 查询仓库列表
     *
     * @param tWarehouse 仓库
     * @return 仓库集合
     */
    public List<TWarehouse> selectTWarehouseList(TWarehouse tWarehouse);

    public List<TWarehouse> selectTWarehouseLists(TWarehouse tWarehouse);

    /**
     * 新增仓库
     *
     * @param tWarehouse 仓库
     * @return 结果
     */
    public int insertTWarehouse(TWarehouse tWarehouse);

    /**
     * 修改仓库
     *
     * @param tWarehouse 仓库
     * @return 结果
     */
    public int updateTWarehouse(TWarehouse tWarehouse);

    /**
     * 删除仓库
     *
     * @param fId 仓库ID
     * @return 结果
     */
    public int deleteTWarehouseById(Long fId);

    /**
     * 批量删除仓库
     *
     * @param fIds 需要删除的数据ID
     * @return 结果
     */
    public int deleteTWarehouseByIds(Long[] fIds);

    /**
     *  检验编号唯一
     * @param fNo
     * @return
     */
    public TWarehouse checkFNoUnique(String fNo);

    /**
     *  检验名称
     * @param fNname
     * @return
     */
    public TWarehouse checkUFNnameUnique(String fNname);

    /**
     * 检验地址
     * @param fAaddr
     * @return
     */
    public TWarehouse checkUFAaddrUnique(String fAaddr);

    /**
     * 根据ID查询所有子部门（正常状态）
     *
     * @param fId 部门ID
     * @return 子部门数
     */
    public int selectNormalChildrenDeptById(Long fId);

    /**
     * 根据ID查询所有子部门
     *
     * @param deptId 部门ID
     * @return 部门列表
     */
    public List<TWarehouse> selectChildrenDeptById(Long deptId);

    /**
     * 修改子元素关系
     *
     * @param depts 子元素
     * @return 结果
     */
    public int updateDeptChildren(@Param("depts") List<TWarehouse> depts);

    /**
     * 修改所在部门的父级部门状态
     *
     * @param tWarehouse 部门
     */
    public void updateDeptStatus(TWarehouse tWarehouse);


    public int updatefTotalgross(TWarehouse tWarehouse);


    /**
     * 查询库区
     *
     * @param fId 库区ID
     * @return 库区
     */
    public TWarehouse selectTWarehousById(Long fId);


    public BigDecimal selectTWarehouseFTotalgross(Long fId);

    public TWarehouse checkDeptNameUnique(@Param("fName") String deptName, @Param("parentId") Long parentId);
}
