package com.ruoyi.warehouseBusiness.service.impl;

import com.ruoyi.warehouseBusiness.domain.BillnoDel;
import com.ruoyi.warehouseBusiness.mapper.BillnoDelMapper;
import com.ruoyi.warehouseBusiness.service.IBillnoDelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 被删除的客户存货编号Service业务层处理
 *
 * @author ruoyi
 * @date 2020-12-23
 */
@Service
public class BillnoDelServiceImpl implements IBillnoDelService {
    @Autowired
    private BillnoDelMapper billnoDelMapper;

    /**
     * 查询被删除的客户存货编号
     *
     * @param id 被删除的客户存货编号ID
     * @return 被删除的客户存货编号
     */
    @Override
    public BillnoDel selectBillnoDelById(Long id) {
        return billnoDelMapper.selectBillnoDelById(id);
    }

    /**
     * 查询被删除的客户存货编号列表
     *
     * @param billnoDel 被删除的客户存货编号
     * @return 被删除的客户存货编号
     */
    @Override
    public List<BillnoDel> selectBillnoDelList(BillnoDel billnoDel) {
        return billnoDelMapper.selectBillnoDelList(billnoDel);
    }

    /**
     * 新增被删除的客户存货编号
     *
     * @param billnoDel 被删除的客户存货编号
     * @return 结果
     */
    @Override
    public int insertBillnoDel(BillnoDel billnoDel) {
        return billnoDelMapper.insertBillnoDel(billnoDel);
    }

    /**
     * 修改被删除的客户存货编号
     *
     * @param billnoDel 被删除的客户存货编号
     * @return 结果
     */
    @Override
    public int updateBillnoDel(BillnoDel billnoDel) {
        return billnoDelMapper.updateBillnoDel(billnoDel);
    }

    /**
     * 批量删除被删除的客户存货编号
     *
     * @param ids 需要删除的被删除的客户存货编号ID
     * @return 结果
     */
    @Override
    public int deleteBillnoDelByIds(Long[] ids) {
        return billnoDelMapper.deleteBillnoDelByIds(ids);
    }

    /**
     * 删除被删除的客户存货编号信息
     *
     * @param id 被删除的客户存货编号ID
     * @return 结果
     */
    @Override
    public int deleteBillnoDelById(Long id) {
        return billnoDelMapper.deleteBillnoDelById(id);
    }
}
